@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/nodesx.c,v 1.50 2009/10/12 17:54:18 amb Exp $
+ $Header: /home/amb/routino/src/RCS/nodesx.c,v 1.53 2009/12/12 11:08:50 amb Exp $
 
  Extented Node data type functions.
 
@@ -36,13 +36,12 @@
 #include "nodes.h"
 
 
-/* Constants */
-
-#define SORT_RAMSIZE (64*1024*1024)
-
 /* Variables */
 
+/*+ The command line '--slim' option. +*/
 extern int option_slim;
+
+/*+ The command line '--tmpdir' option or its default value. +*/
 extern char *option_tmpdirname;
 
 /*+ A temporary file-local variable for use by the sort functions. +*/
@@ -173,7 +172,7 @@ void SortNodeList(NodesX* nodesx)
 
  sortnodesx=nodesx;
 
- filesort(nodesx->fd,fd,sizeof(NodeX),SORT_RAMSIZE,(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))index_by_id);
+ filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))index_by_id);
 
  /* Close the files and re-open them */
 
@@ -216,7 +215,7 @@ static int sort_by_id(NodeX *a,NodeX *b)
 /*++++++++++++++++++++++++++++++++++++++
   Index the nodes after sorting.
 
-  index_by_id Return 1 if the value is to be kept, otherwise zero.
+  int index_by_id Return 1 if the value is to be kept, otherwise zero.
 
   NodeX *nodex The extended node.
 
@@ -272,7 +271,7 @@ void SortNodeListGeographically(NodesX* nodesx)
 
  sortnodesx=nodesx;
 
- filesort(nodesx->fd,fd,sizeof(NodeX),SORT_RAMSIZE,(int (*)(const void*,const void*))sort_by_lat_long,(int (*)(void*,index_t))index_by_lat_long);
+ filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_lat_long,(int (*)(void*,index_t))index_by_lat_long);
 
  /* Close the files and re-open them */
 
@@ -345,7 +344,7 @@ static int sort_by_lat_long(NodeX *a,NodeX *b)
 /*++++++++++++++++++++++++++++++++++++++
   Index the nodes after sorting.
 
-  index_by_lat_long Return 1 if the value is to be kept, otherwise zero.
+  int index_by_lat_long Return 1 if the value is to be kept, otherwise zero.
 
   NodeX *nodex The extended node.
 
