@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/optimiser.c,v 1.76 2009/08/15 15:27:47 amb Exp $
+ $Header: /home/amb/routino/src/RCS/optimiser.c,v 1.82 2009/11/25 15:00:37 amb Exp $
 
  Routing optimiser.
 
@@ -72,7 +72,10 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t star
 
  finish_score=INF_SCORE;
 
- GetLatLong(nodes,finish,&finish_lat,&finish_lon);
+ if(IsFakeNode(finish))
+    GetFakeLatLong(finish,&finish_lat,&finish_lon);
+ else
+    GetLatLong(nodes,finish,&finish_lat,&finish_lon);
 
  /* Create the list of results and insert the first node into the queue */
 
@@ -98,11 +101,17 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t star
 
     node1=result1->node;
 
-    segment=FirstSegment(segments,nodes,node1);
+    if(IsFakeNode(node1))
+       segment=FirstFakeSegment(node1);
+    else
+       segment=FirstSegment(segments,nodes,node1);
 
     while(segment)
       {
-       score_t segment_score,cumulative_score;
+       score_t segment_pref,segment_score,cumulative_score;
+       int i;
+
+       node2=OtherNode(segment,node1);  /* need this here because we use node2 later */
 
        if(!IsNormalSegment(segment))
           goto endloop;
@@ -110,12 +119,10 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t star
        if(profile->oneway && IsOnewayTo(segment,node1))
           goto endloop;
 
-       node2=OtherNode(segment,node1);
-
        if(result1->prev==node2)
           goto endloop;
 
-       if(node2!=finish && IsSuperNode(nodes,node2))
+       if(node2!=finish && !IsFakeNode(node2) && IsSuperNode(nodes,node2))
           goto endloop;
 
        way=LookupWay(ways,segment->way);
@@ -134,10 +141,24 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t star
           (way->length && way->length<profile->length))
           goto endloop;
 
+       segment_pref=profile->highway[HIGHWAY(way->type)];
+
+       for(i=1;i<Property_Count;i++)
+          if(ways->props && PROPERTIES(i))
+            {
+             if(way->props && PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
+
+       if(segment_pref==0)
+          goto endloop;
+
        if(option_quickest==0)
-          segment_score=(score_t)DISTANCE(segment->distance)/profile->highway[HIGHWAY(way->type)];
+          segment_score=(score_t)DISTANCE(segment->distance)/segment_pref;
        else
-          segment_score=(score_t)Duration(segment,way,profile)/profile->highway[HIGHWAY(way->type)];
+          segment_score=(score_t)Duration(segment,way,profile)/segment_pref;
 
        cumulative_score=result1->score+segment_score;
 
@@ -186,7 +207,17 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t star
 
       endloop:
 
-       segment=NextSegment(segments,segment,node1);
+       if(IsFakeNode(node1))
+          segment=NextFakeSegment(segment,node1);
+       else if(IsFakeNode(node2))
+          segment=NULL; /* cannot call NextSegment() with a fake segment */
+       else
+         {
+          segment=NextSegment(segments,segment,node1);
+
+          if(!segment && IsFakeNode(finish))
+             segment=ExtraFakeSegment(node1,finish);
+         }
       }
    }
 
@@ -194,34 +225,13 @@ Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t star
 
  /* Check it worked */
 
- result2=FindResult(results,finish);
-
- if(!result2)
+ if(!FindResult(results,finish))
    {
     FreeResultsList(results);
     return(NULL);
    }
 
- /* Create the forward links for the optimum path */
-
- result2=FindResult(results,finish);
-
- do
-   {
-    if(result2->prev!=NO_NODE)
-      {
-       index_t node1=result2->prev;
-
-       result1=FindResult(results,node1);
-
-       result1->next=result2->node;
-
-       result2=result1;
-      }
-    else
-       result2=NULL;
-   }
- while(result2);
+ FixForwardRoute(results,finish);
 
  return(results);
 }
@@ -258,15 +268,18 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
 
  if(!option_quiet)
    {
-    printf("Routing: End Nodes=0");
+    printf("Routing: Super-Nodes checked = 0");
     fflush(stdout);
    }
 
  /* Set up the finish conditions */
 
- finish_score=~(distance_t)0;
+ finish_score=INF_DISTANCE;
 
- GetLatLong(nodes,end->finish,&finish_lat,&finish_lon);
+ if(IsFakeNode(end->finish))
+    GetFakeLatLong(end->finish,&finish_lat,&finish_lon);
+ else
+    GetLatLong(nodes,end->finish,&finish_lat,&finish_lon);
 
  /* Create the list of results and insert the first node into the queue */
 
@@ -288,7 +301,7 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
 
  while(result3)
    {
-    if(result3->node!=begin->start && IsSuperNode(nodes,result3->node))
+    if(result3->node!=begin->start && !IsFakeNode(result3->node) && IsSuperNode(nodes,result3->node))
       {
        result2=InsertResult(results,result3->node);
 
@@ -316,11 +329,15 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
 
     node1=result1->node;
 
-    segment=FirstSegment(segments,nodes,node1);
+    if(IsFakeNode(node1))
+       segment=FirstFakeSegment(node1);
+    else
+       segment=FirstSegment(segments,nodes,node1);
 
     while(segment)
       {
-       score_t segment_score,cumulative_score;
+       score_t segment_pref,segment_score,cumulative_score;
+       int i;
 
        if(!IsSuperSegment(segment))
           goto endloop;
@@ -349,10 +366,24 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
           (way->length && way->length<profile->length))
           goto endloop;
 
+       segment_pref=profile->highway[HIGHWAY(way->type)];
+
+       for(i=1;i<Property_Count;i++)
+          if(ways->props && PROPERTIES(i))
+            {
+             if(way->props && PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
+
+       if(segment_pref==0)
+          goto endloop;
+
        if(option_quickest==0)
-          segment_score=(score_t)DISTANCE(segment->distance)/profile->highway[HIGHWAY(way->type)];
+          segment_score=(score_t)DISTANCE(segment->distance)/segment_pref;
        else
-          segment_score=(score_t)Duration(segment,way,profile)/profile->highway[HIGHWAY(way->type)];
+          segment_score=(score_t)Duration(segment,way,profile)/segment_pref;
 
        cumulative_score=result1->score+segment_score;
 
@@ -378,7 +409,10 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
              double lat,lon;
              distance_t direct;
 
-             GetLatLong(nodes,node2,&lat,&lon);
+             if(IsFakeNode(node2))
+                GetFakeLatLong(node2,&lat,&lon);
+             else
+                GetLatLong(nodes,node2,&lat,&lon);
              direct=Distance(lat,lon,finish_lat,finish_lon);
 
              if(option_quickest==0)
@@ -407,7 +441,10 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
              double lat,lon;
              distance_t direct;
 
-             GetLatLong(nodes,node2,&lat,&lon);
+             if(IsFakeNode(node2))
+                GetFakeLatLong(node2,&lat,&lon);
+             else
+                GetLatLong(nodes,node2,&lat,&lon);
              direct=Distance(lat,lon,finish_lat,finish_lon);
 
              if(option_quickest==0)
@@ -423,17 +460,20 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
 
        if(!option_quiet && !(results->number%10000))
          {
-          printf("\rRouting: End Nodes=%d",results->number);
+          printf("\rRouting: Super-Nodes checked = %d",results->number);
           fflush(stdout);
          }
 
-       segment=NextSegment(segments,segment,node1);
+       if(IsFakeNode(node1))
+          segment=NextFakeSegment(segment,node1);
+       else
+          segment=NextSegment(segments,segment,node1);
       }
    }
 
  if(!option_quiet)
    {
-    printf("\rRouted: End Super-Nodes=%d\n",results->number);
+    printf("\rRouted: Super-Nodes checked = %d\n",results->number);
     fflush(stdout);
    }
 
@@ -446,13 +486,13 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
 
     *result2=*result3;
 
-    result2->score=~(distance_t)0;
+    result2->score=INF_DISTANCE;
 
     result3=FirstResult(end);
 
     while(result3)
       {
-       if(IsSuperNode(nodes,result3->node))
+       if(!IsFakeNode(result3->node) && IsSuperNode(nodes,result3->node))
           if((result1=FindResult(results,result3->node)))
              if((result1->score+result3->score)<result2->score)
                {
@@ -468,32 +508,13 @@ Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *beg
 
  /* Check it worked */
 
- result2=FindResult(results,end->finish);
-
- if(!result2)
+ if(!FindResult(results,end->finish))
    {
     FreeResultsList(results);
     return(NULL);
    }
 
- /* Create the forward links for the optimum path */
-
- do
-   {
-    if(result2->prev!=NO_NODE)
-      {
-       index_t node1=result2->prev;
-
-       result1=FindResult(results,node1);
-
-       result1->next=result2->node;
-
-       result2=result1;
-      }
-    else
-       result2=NULL;
-   }
- while(result2);
+ FixForwardRoute(results,end->finish);
 
  return(results);
 }
@@ -544,11 +565,15 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
    {
     node1=result1->node;
 
-    segment=FirstSegment(segments,nodes,node1);
+    if(IsFakeNode(node1))
+       segment=FirstFakeSegment(node1);
+    else
+       segment=FirstSegment(segments,nodes,node1);
 
     while(segment)
       {
-       score_t segment_score,cumulative_score;
+       score_t segment_pref,segment_score,cumulative_score;
+       int i;
 
        if(!IsNormalSegment(segment))
           goto endloop;
@@ -577,10 +602,24 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
           (way->length && way->length<profile->length))
           goto endloop;
 
+       segment_pref=profile->highway[HIGHWAY(way->type)];
+
+       for(i=1;i<Property_Count;i++)
+          if(ways->props && PROPERTIES(i))
+            {
+             if(way->props && PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
+
+       if(segment_pref==0)
+          goto endloop;
+
        if(option_quickest==0)
-          segment_score=(score_t)DISTANCE(segment->distance)/profile->highway[HIGHWAY(way->type)];
+          segment_score=(score_t)DISTANCE(segment->distance)/segment_pref;
        else
-          segment_score=(score_t)Duration(segment,way,profile)/profile->highway[HIGHWAY(way->type)];
+          segment_score=(score_t)Duration(segment,way,profile)/segment_pref;
 
        cumulative_score=result1->score+segment_score;
 
@@ -594,7 +633,7 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
           result2->score=cumulative_score;
           result2->segment=segment;
 
-          if(!IsSuperNode(nodes,node2))
+          if(!IsFakeNode(node2) && !IsSuperNode(nodes,node2))
             {
              result2->sortby=result2->score;
              InsertInQueue(queue,result2);
@@ -606,7 +645,7 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
           result2->score=cumulative_score;
           result2->segment=segment;
 
-          if(!IsSuperNode(nodes,node2))
+          if(!IsFakeNode(node2) && !IsSuperNode(nodes,node2))
             {
              result2->sortby=result2->score;
              InsertInQueue(queue,result2);
@@ -615,7 +654,10 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
 
       endloop:
 
-       segment=NextSegment(segments,segment,node1);
+       if(IsFakeNode(node1))
+          segment=NextFakeSegment(segment,node1);
+       else
+          segment=NextSegment(segments,segment,node1);
       }
    }
 
@@ -678,11 +720,15 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
    {
     node1=result1->node;
 
-    segment=FirstSegment(segments,nodes,node1);
+    if(IsFakeNode(node1))
+       segment=FirstFakeSegment(node1);
+    else
+       segment=FirstSegment(segments,nodes,node1);
 
     while(segment)
       {
-       score_t segment_score,cumulative_score;
+       score_t segment_pref,segment_score,cumulative_score;
+       int i;
 
        if(!IsNormalSegment(segment))
           goto endloop;
@@ -711,10 +757,24 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
           (way->length && way->length<profile->length))
           goto endloop;
 
+       segment_pref=profile->highway[HIGHWAY(way->type)];
+
+       for(i=1;i<Property_Count;i++)
+          if(ways->props && PROPERTIES(i))
+            {
+             if(way->props && PROPERTIES(i))
+                segment_pref*=profile->props_yes[i];
+             else
+                segment_pref*=profile->props_no[i];
+            }
+
+       if(segment_pref==0)
+          goto endloop;
+
        if(option_quickest==0)
-          segment_score=(score_t)DISTANCE(segment->distance)/profile->highway[HIGHWAY(way->type)];
+          segment_score=(score_t)DISTANCE(segment->distance)/segment_pref;
        else
-          segment_score=(score_t)Duration(segment,way,profile)/profile->highway[HIGHWAY(way->type)];
+          segment_score=(score_t)Duration(segment,way,profile)/segment_pref;
 
        cumulative_score=result1->score+segment_score;
 
@@ -728,7 +788,7 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
           result2->score=cumulative_score;
           result2->segment=segment;
 
-          if(!IsSuperNode(nodes,node2))
+          if(!IsFakeNode(node2) && !IsSuperNode(nodes,node2))
             {
              result2->sortby=result2->score;
              InsertInQueue(queue,result2);
@@ -740,7 +800,7 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
           result2->score=cumulative_score;
           result2->segment=segment;
 
-          if(!IsSuperNode(nodes,node2))
+          if(!IsFakeNode(node2) && !IsSuperNode(nodes,node2))
             {
              result2->sortby=result2->score;
              InsertInQueue(queue,result2);
@@ -749,7 +809,10 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
 
       endloop:
 
-       segment=NextSegment(segments,segment,node1);
+       if(IsFakeNode(node1))
+          segment=NextFakeSegment(segment,node1);
+       else
+          segment=NextSegment(segments,segment,node1);
       }
    }
 
@@ -840,3 +903,39 @@ Results *CombineRoutes(Results *results,Nodes *nodes,Segments *segments,Ways *wa
 
  return(combined);
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Fx the forward route (i.e. setup next nodes for forward path from prev nodes on reverse path).
+
+  Results *results The set of results to update.
+
+  index_t finish The finish point.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void FixForwardRoute(Results *results,index_t finish)
+{
+ Result *result2=FindResult(results,finish);
+ Result *result1;
+
+ /* Create the forward links for the optimum path */
+
+ do
+   {
+    if(result2->prev!=NO_NODE)
+      {
+       index_t node1=result2->prev;
+
+       result1=FindResult(results,node1);
+
+       result1->next=result2->node;
+
+       result2=result1;
+      }
+    else
+       result2=NULL;
+   }
+ while(result2);
+
+ results->finish=finish;
+}
