@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/profiles.c,v 1.18 2009/07/12 09:01:48 amb Exp $
+ $Header: /home/amb/routino/src/RCS/profiles.c,v 1.29 2009/12/15 18:44:28 amb Exp $
 
  The pre-defined profiles and the functions for handling them.
 
@@ -47,10 +47,9 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] =  90,
                                    [Way_Service     ] =  90,
                                    [Way_Track       ] =  95,
-                                   [Way_Path        ] = 100,
-                                   [Way_Bridleway   ] = 100,
                                    [Way_Cycleway    ] =  95,
-                                   [Way_Footway     ] = 100,
+                                   [Way_Path        ] = 100,
+                                   [Way_Steps       ] =  80,
                                   },
                       .speed    = {
                                    [Way_Motorway    ] = 0,
@@ -62,10 +61,107 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] = kph_to_speed(4),
                                    [Way_Service     ] = kph_to_speed(4),
                                    [Way_Track       ] = kph_to_speed(4),
-                                   [Way_Path        ] = kph_to_speed(4),
-                                   [Way_Bridleway   ] = kph_to_speed(4),
                                    [Way_Cycleway    ] = kph_to_speed(4),
-                                   [Way_Footway     ] = kph_to_speed(4),
+                                   [Way_Path        ] = kph_to_speed(4),
+                                   [Way_Steps       ] = kph_to_speed(4),
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 50,
+                                   [Property_Multilane] = 25,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
+                                  },
+                      .oneway   = 0,
+                      .weight   = 0,
+                      .height   = 0,
+                      .width    = 0,
+                      .length   = 0,
+                     },
+
+  /* The profile for travel by Horse */
+
+  [Transport_Horse] = {
+                      .transport=Transport_Horse,
+                      .allow    =Allow_Horse,
+                      .highway  = {
+                                   [Way_Motorway    ] =   0,
+                                   [Way_Trunk       ] =  25,
+                                   [Way_Primary     ] =  50,
+                                   [Way_Secondary   ] =  50,
+                                   [Way_Tertiary    ] =  75,
+                                   [Way_Unclassified] =  75,
+                                   [Way_Residential ] =  75,
+                                   [Way_Service     ] =  75,
+                                   [Way_Track       ] = 100,
+                                   [Way_Cycleway    ] =  90,
+                                   [Way_Path        ] = 100,
+                                   [Way_Steps       ] =   0,
+                                  },
+                      .speed    = {
+                                   [Way_Motorway    ] = 0,
+                                   [Way_Trunk       ] = kph_to_speed(8),
+                                   [Way_Primary     ] = kph_to_speed(8),
+                                   [Way_Secondary   ] = kph_to_speed(8),
+                                   [Way_Tertiary    ] = kph_to_speed(8),
+                                   [Way_Unclassified] = kph_to_speed(8),
+                                   [Way_Residential ] = kph_to_speed(8),
+                                   [Way_Service     ] = kph_to_speed(8),
+                                   [Way_Track       ] = kph_to_speed(8),
+                                   [Way_Cycleway    ] = kph_to_speed(8),
+                                   [Way_Path        ] = kph_to_speed(8),
+                                   [Way_Steps       ] = 0,
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 20,
+                                   [Property_Multilane] = 25,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
+                                  },
+                      .oneway   = 1,
+                      .weight   = 0,
+                      .height   = 0,
+                      .width    = 0,
+                      .length   = 0,
+                     },
+
+  /* The profile for travel by Wheelchair */
+
+  [Transport_Wheelchair] = {
+                      .transport=Transport_Wheelchair,
+                      .allow    =Allow_Wheelchair,
+                      .highway  = {
+                                   [Way_Motorway    ] =   0,
+                                   [Way_Trunk       ] =  40,
+                                   [Way_Primary     ] =  50,
+                                   [Way_Secondary   ] =  60,
+                                   [Way_Tertiary    ] =  70,
+                                   [Way_Unclassified] =  80,
+                                   [Way_Residential ] =  90,
+                                   [Way_Service     ] =  90,
+                                   [Way_Track       ] =  95,
+                                   [Way_Cycleway    ] =  95,
+                                   [Way_Path        ] = 100,
+                                   [Way_Steps       ] =   0,
+                                  },
+                      .speed    = {
+                                   [Way_Motorway    ] = 0,
+                                   [Way_Trunk       ] = kph_to_speed(4),
+                                   [Way_Primary     ] = kph_to_speed(4),
+                                   [Way_Secondary   ] = kph_to_speed(4),
+                                   [Way_Tertiary    ] = kph_to_speed(4),
+                                   [Way_Unclassified] = kph_to_speed(4),
+                                   [Way_Residential ] = kph_to_speed(4),
+                                   [Way_Service     ] = kph_to_speed(4),
+                                   [Way_Track       ] = kph_to_speed(4),
+                                   [Way_Cycleway    ] = kph_to_speed(4),
+                                   [Way_Path        ] = kph_to_speed(4),
+                                   [Way_Steps       ] = kph_to_speed(4),
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 90,
+                                   [Property_Multilane] = 25,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
                                   },
                       .oneway   = 0,
                       .weight   = 0,
@@ -89,14 +185,13 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] =  90,
                                    [Way_Service     ] =  90,
                                    [Way_Track       ] =  90,
-                                   [Way_Path        ] =  90,
-                                   [Way_Bridleway   ] =  90,
                                    [Way_Cycleway    ] = 100,
-                                   [Way_Footway     ] =  90,
+                                   [Way_Path        ] =  90,
+                                   [Way_Steps       ] =   0,
                                   },
                       .speed    = {
                                    [Way_Motorway    ] = 0,
-                                   [Way_Trunk       ] = 0,
+                                   [Way_Trunk       ] = kph_to_speed(20),
                                    [Way_Primary     ] = kph_to_speed(20),
                                    [Way_Secondary   ] = kph_to_speed(20),
                                    [Way_Tertiary    ] = kph_to_speed(20),
@@ -104,10 +199,15 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] = kph_to_speed(20),
                                    [Way_Service     ] = kph_to_speed(20),
                                    [Way_Track       ] = kph_to_speed(20),
-                                   [Way_Path        ] = kph_to_speed(20),
-                                   [Way_Bridleway   ] = kph_to_speed(20),
                                    [Way_Cycleway    ] = kph_to_speed(20),
-                                   [Way_Footway     ] = kph_to_speed(20),
+                                   [Way_Path        ] = kph_to_speed(20),
+                                   [Way_Steps       ] = 0,
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 50,
+                                   [Property_Multilane] = 25,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
                                   },
                       .oneway   = 1,
                       .weight   = 0,
@@ -116,40 +216,44 @@ static Profile builtin_profiles[]=
                       .length   = 0,
                      },
 
-  /* The profile for travel by Horse */
+  /* The profile for travel by Moped */
 
-  [Transport_Horse] = {
-                      .transport=Transport_Horse,
-                      .allow    =Allow_Horse,
+  [Transport_Moped] = {
+                      .transport=Transport_Moped,
+                      .allow    =Allow_Moped,
                       .highway  = {
                                    [Way_Motorway    ] =   0,
-                                   [Way_Trunk       ] =  25,
-                                   [Way_Primary     ] =  50,
-                                   [Way_Secondary   ] =  50,
-                                   [Way_Tertiary    ] =  75,
-                                   [Way_Unclassified] =  75,
-                                   [Way_Residential ] =  75,
-                                   [Way_Service     ] =  75,
-                                   [Way_Track       ] = 100,
-                                   [Way_Path        ] = 100,
-                                   [Way_Bridleway   ] = 100,
-                                   [Way_Cycleway    ] =  90,
-                                   [Way_Footway     ] =  90,
+                                   [Way_Trunk       ] =  90,
+                                   [Way_Primary     ] = 100,
+                                   [Way_Secondary   ] =  90,
+                                   [Way_Tertiary    ] =  80,
+                                   [Way_Unclassified] =  70,
+                                   [Way_Residential ] =  60,
+                                   [Way_Service     ] =  80,
+                                   [Way_Track       ] =   0,
+                                   [Way_Cycleway    ] =   0,
+                                   [Way_Path        ] =   0,
+                                   [Way_Steps       ] =   0,
                                   },
                       .speed    = {
-                                   [Way_Motorway    ] = 0,
-                                   [Way_Trunk       ] = 0,
-                                   [Way_Primary     ] = kph_to_speed(8),
-                                   [Way_Secondary   ] = kph_to_speed(8),
-                                   [Way_Tertiary    ] = kph_to_speed(8),
-                                   [Way_Unclassified] = kph_to_speed(8),
-                                   [Way_Residential ] = kph_to_speed(8),
-                                   [Way_Service     ] = kph_to_speed(8),
-                                   [Way_Track       ] = kph_to_speed(8),
-                                   [Way_Path        ] = kph_to_speed(8),
-                                   [Way_Bridleway   ] = kph_to_speed(8),
-                                   [Way_Cycleway    ] = kph_to_speed(8),
-                                   [Way_Footway     ] = kph_to_speed(8),
+                                   [Way_Motorway    ] = kph_to_speed(30*1.6),
+                                   [Way_Trunk       ] = kph_to_speed(30*1.6),
+                                   [Way_Primary     ] = kph_to_speed(30*1.6),
+                                   [Way_Secondary   ] = kph_to_speed(30*1.6),
+                                   [Way_Tertiary    ] = kph_to_speed(30*1.6),
+                                   [Way_Unclassified] = kph_to_speed(30*1.6),
+                                   [Way_Residential ] = kph_to_speed(30*1.6),
+                                   [Way_Service     ] = kph_to_speed(20*1.6),
+                                   [Way_Track       ] = kph_to_speed(10*1.6),
+                                   [Way_Cycleway    ] = 0,
+                                   [Way_Path        ] = 0,
+                                   [Way_Steps       ] = 0,
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 100,
+                                   [Property_Multilane] = 25,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
                                   },
                       .oneway   = 1,
                       .weight   = 0,
@@ -173,10 +277,9 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] =  50,
                                    [Way_Service     ] =  80,
                                    [Way_Track       ] =   0,
-                                   [Way_Path        ] =   0,
-                                   [Way_Bridleway   ] =   0,
                                    [Way_Cycleway    ] =   0,
-                                   [Way_Footway     ] =   0,
+                                   [Way_Path        ] =   0,
+                                   [Way_Steps       ] =   0,
                                   },
                       .speed    = {
                                    [Way_Motorway    ] = kph_to_speed(70*1.6),
@@ -188,10 +291,15 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] = kph_to_speed(30*1.6),
                                    [Way_Service     ] = kph_to_speed(20*1.6),
                                    [Way_Track       ] = kph_to_speed(10*1.6),
-                                   [Way_Path        ] = 0,
-                                   [Way_Bridleway   ] = 0,
                                    [Way_Cycleway    ] = 0,
-                                   [Way_Footway     ] = 0,
+                                   [Way_Path        ] = 0,
+                                   [Way_Steps       ] = 0,
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 100,
+                                   [Property_Multilane] = 75,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
                                   },
                       .oneway   = 1,
                       .weight   = 0,
@@ -215,10 +323,9 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] =  50,
                                    [Way_Service     ] =  80,
                                    [Way_Track       ] =   0,
-                                   [Way_Path        ] =   0,
-                                   [Way_Bridleway   ] =   0,
                                    [Way_Cycleway    ] =   0,
-                                   [Way_Footway     ] =   0,
+                                   [Way_Path        ] =   0,
+                                   [Way_Steps       ] =   0,
                                   },
                       .speed    = {
                                    [Way_Motorway    ] = kph_to_speed(70*1.6),
@@ -230,10 +337,15 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] = kph_to_speed(30*1.6),
                                    [Way_Service     ] = kph_to_speed(20*1.6),
                                    [Way_Track       ] = kph_to_speed(10*1.6),
-                                   [Way_Path        ] = 0,
-                                   [Way_Bridleway   ] = 0,
                                    [Way_Cycleway    ] = 0,
-                                   [Way_Footway     ] = 0,
+                                   [Way_Path        ] = 0,
+                                   [Way_Steps       ] = 0,
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 100,
+                                   [Property_Multilane] = 75,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
                                   },
                       .oneway   = 1,
                       .weight   = 0,
@@ -257,10 +369,9 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] =  50,
                                    [Way_Service     ] =  80,
                                    [Way_Track       ] =   0,
-                                   [Way_Path        ] =   0,
-                                   [Way_Bridleway   ] =   0,
                                    [Way_Cycleway    ] =   0,
-                                   [Way_Footway     ] =   0,
+                                   [Way_Path        ] =   0,
+                                   [Way_Steps       ] =   0,
                                   },
                       .speed    = {
                                    [Way_Motorway    ] = kph_to_speed(60*1.6),
@@ -272,10 +383,15 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] = kph_to_speed(30*1.6),
                                    [Way_Service     ] = kph_to_speed(20*1.6),
                                    [Way_Track       ] = kph_to_speed(10*1.6),
-                                   [Way_Path        ] = 0,
-                                   [Way_Bridleway   ] = 0,
                                    [Way_Cycleway    ] = 0,
-                                   [Way_Footway     ] = 0,
+                                   [Way_Path        ] = 0,
+                                   [Way_Steps       ] = 0,
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 100,
+                                   [Property_Multilane] = 75,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
                                   },
                       .oneway   = 1,
                       .weight   = tonnes_to_weight(5),
@@ -299,10 +415,9 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] =  50,
                                    [Way_Service     ] =  80,
                                    [Way_Track       ] =   0,
-                                   [Way_Path        ] =   0,
-                                   [Way_Bridleway   ] =   0,
                                    [Way_Cycleway    ] =   0,
-                                   [Way_Footway     ] =   0,
+                                   [Way_Path        ] =   0,
+                                   [Way_Steps       ] =   0,
                                   },
                       .speed    = {
                                    [Way_Motorway    ] = kph_to_speed(56*1.6),
@@ -314,10 +429,15 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] = kph_to_speed(30*1.6),
                                    [Way_Service     ] = kph_to_speed(20*1.6),
                                    [Way_Track       ] = kph_to_speed(10*1.6),
-                                   [Way_Path        ] = 0,
-                                   [Way_Bridleway   ] = 0,
                                    [Way_Cycleway    ] = 0,
-                                   [Way_Footway     ] = 0,
+                                   [Way_Path        ] = 0,
+                                   [Way_Steps       ] = 0,
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 100,
+                                   [Property_Multilane] = 75,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
                                   },
                       .oneway   = 1,
                       .weight   = tonnes_to_weight(10),
@@ -341,10 +461,9 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] =  50,
                                    [Way_Service     ] =  80,
                                    [Way_Track       ] =   0,
-                                   [Way_Path        ] =   0,
-                                   [Way_Bridleway   ] =   0,
                                    [Way_Cycleway    ] =   0,
-                                   [Way_Footway     ] =   0,
+                                   [Way_Path        ] =   0,
+                                   [Way_Steps       ] =   0,
                                   },
                       .speed    = {
                                    [Way_Motorway    ] = kph_to_speed(56*1.6),
@@ -356,10 +475,15 @@ static Profile builtin_profiles[]=
                                    [Way_Residential ] = kph_to_speed(30*1.6),
                                    [Way_Service     ] = kph_to_speed(20*1.6),
                                    [Way_Track       ] = kph_to_speed(10*1.6),
-                                   [Way_Path        ] = 0,
-                                   [Way_Bridleway   ] = 0,
                                    [Way_Cycleway    ] = 0,
-                                   [Way_Footway     ] = 0,
+                                   [Way_Path        ] = 0,
+                                   [Way_Steps       ] = 0,
+                                  },
+                      .props_yes= {
+                                   [Property_Paved    ] = 100,
+                                   [Property_Multilane] = 75,
+                                   [Property_Bridge   ] = 50,
+                                   [Property_Tunnel   ] = 50,
                                   },
                       .oneway   = 1,
                       .weight   = tonnes_to_weight(15),
@@ -397,29 +521,47 @@ void UpdateProfile(Profile *profile)
 
  /* Normalise the highway preferences into the range 0 -> 1 */
 
- for(i=1;i<Way_Unknown;i++)
+ for(i=1;i<Way_Count;i++)
+   {
+    if(profile->highway[i]<0)
+       profile->highway[i]=0;
+
     if(profile->highway[i]>hmax)
        hmax=profile->highway[i];
+   }
 
- for(i=1;i<Way_Unknown;i++)
-    if(profile->highway[i]>0)
-       profile->highway[i]/=hmax;
-    else
-       profile->highway[i]=0;
+ for(i=1;i<Way_Count;i++)
+    profile->highway[i]/=hmax;
+
+ /* Normalise the attribute preferences into the range 0 -> 1 */
+
+ for(i=1;i<Property_Count;i++)
+   {
+    if(profile->props_yes[i]<0)
+       profile->props_yes[i]=0;
+
+    if(profile->props_yes[i]>100)
+       profile->props_yes[i]=100;
+
+    profile->props_yes[i]/=100;
+    profile->props_no [i] =1-profile->props_yes[i];
+   }
 
  /* Find the fastest and most preferred highway type */
 
  profile->max_speed=0;
 
- for(i=0;i<Way_Unknown;i++)
+ for(i=1;i<Way_Count;i++)
     if(profile->speed[i]>profile->max_speed)
        profile->max_speed=profile->speed[i];
 
- profile->max_pref=0;
+ profile->max_pref=1; /* since highway prefs were normalised to 1 */
 
- for(i=0;i<Way_Unknown;i++)
-    if(profile->highway[i]>profile->max_pref)
-       profile->max_pref=profile->highway[i];
+ for(i=1;i<Property_Count;i++)
+    if(profile->props_yes[i]>profile->props_no[i])
+       profile->max_pref*=profile->props_yes[i];
+    else if(profile->props_no[i]>profile->props_yes[i])
+       profile->max_pref*=profile->props_no[i];
 }
 
 
@@ -441,17 +583,22 @@ void PrintProfile(const Profile *profile)
 
  printf("\n");
 
- for(i=1;i<Way_Unknown;i++)
+ for(i=1;i<Way_Count;i++)
     printf("Highway %-12s: %3d%%\n",HighwayName(i),(int)profile->highway[i]);
 
  printf("\n");
 
- for(i=1;i<Way_Unknown;i++)
+ for(i=1;i<Way_Count;i++)
     if(profile->highway[i])
        printf("Speed on %-12s: %3d km/h / %2.0f mph\n",HighwayName(i),profile->speed[i],(double)profile->speed[i]/1.6);
 
  printf("\n");
 
+ for(i=1;i<Property_Count;i++)
+    printf("Highway property %-12s: %3d%%\n",PropertyName(i),(int)profile->props_yes[i]);
+
+ printf("\n");
+
  printf("Obey one-way  : %s\n",profile->oneway?"yes":"no");
  printf("Minimum weight: %.1f tonnes\n",weight_to_tonnes(profile->weight));
  printf("Minimum height: %.1f metres\n",height_to_metres(profile->height));
@@ -471,14 +618,21 @@ void PrintProfilesJS(void)
  printf("// Transport types\n");
  printf("var router_transports={");
  for(j=1;j<sizeof(builtin_profiles)/sizeof(builtin_profiles[0]);j++)
-    printf("%s%s: %d",j==1?"":", ",TransportName(j),j-1);
+    printf("%s%s: %d",j==1?"":", ",TransportName(j),j);
  printf("};\n");
  printf("\n");
 
  printf("// Highway types\n");
  printf("var router_highways={");
- for(i=1;i<Way_Unknown;i++)
-    printf("%s%s: %d",i==1?"":", ",HighwayName(i),i-1);
+ for(i=1;i<Way_Count;i++)
+    printf("%s%s: %d",i==1?"":", ",HighwayName(i),i);
+ printf("};\n");
+ printf("\n");
+
+ printf("// Property types\n");
+ printf("var router_properties={");
+ for(i=1;i<Property_Count;i++)
+    printf("%s%s: %d",i==1?"":", ",PropertyName(i),i);
  printf("};\n");
  printf("\n");
 
@@ -488,24 +642,36 @@ void PrintProfilesJS(void)
 
  printf("// Allowed highways\n");
  printf("var router_profile_highway={\n");
- for(i=1;i<Way_Unknown;i++)
+ for(i=1;i<Way_Count;i++)
    {
     printf("  %12s: {",HighwayName(i));
     for(j=1;j<sizeof(builtin_profiles)/sizeof(builtin_profiles[0]);j++)
        printf("%s%s: %3d",j==1?"":", ",TransportName(j),(int)builtin_profiles[j].highway[i]);
-    printf("}%s\n",i==(Way_Unknown-1)?"":", ");
+    printf("}%s\n",i==(Way_Count-1)?"":",");
    }
  printf("   };\n");
  printf("\n");
 
  printf("// Speed limits\n");
  printf("var router_profile_speed={\n");
- for(i=1;i<Way_Unknown;i++)
+ for(i=1;i<Way_Count;i++)
    {
     printf("  %12s: {",HighwayName(i));
     for(j=1;j<sizeof(builtin_profiles)/sizeof(builtin_profiles[0]);j++)
        printf("%s%s: %3d",j==1?"":", ",TransportName(j),builtin_profiles[j].speed[i]);
-    printf("}%s\n",i==(Way_Unknown-1)?"":", ");
+    printf("}%s\n",i==(Way_Count-1)?"":",");
+   }
+ printf("   };\n");
+ printf("\n");
+
+ printf("// Highway properties\n");
+ printf("var router_profile_property={\n");
+ for(i=1;i<Property_Count;i++)
+   {
+    printf("  %12s: {",PropertyName(i));
+    for(j=1;j<sizeof(builtin_profiles)/sizeof(builtin_profiles[0]);j++)
+       printf("%s%s: %3d",j==1?"":", ",TransportName(j),(int)builtin_profiles[j].props_yes[i]);
+    printf("}%s\n",i==(Property_Count-1)?"":",");
    }
  printf("   };\n");
  printf("\n");
@@ -554,35 +720,54 @@ void PrintProfilesPerl(void)
 
  printf("# Highway types\n");
  printf("@router_highways=(");
- for(i=1;i<Way_Unknown;i++)
+ for(i=1;i<Way_Count;i++)
     printf("%s'%s'",i==1?"":", ",HighwayName(i));
  printf(");\n");
  printf("\n");
 
+ printf("# Property types\n");
+ printf("@router_properties=(");
+ for(i=1;i<Property_Count;i++)
+    printf("%s'%s'",i==1?"":", ",PropertyName(i));
+ printf(");\n");
+ printf("\n");
+
  printf("# Restriction types\n");
  printf("@router_restrictions=('oneway', 'weight', 'height', 'width', 'length');\n");
  printf("\n");
 
  printf("# Allowed highways\n");
  printf("%%router_profile_highway=(\n");
- for(i=1;i<Way_Unknown;i++)
+ for(i=1;i<Way_Count;i++)
    {
     printf("  %12s => {",HighwayName(i));
     for(j=1;j<sizeof(builtin_profiles)/sizeof(builtin_profiles[0]);j++)
        printf("%s %s => %3d",j==1?"":", ",TransportName(j),(int)builtin_profiles[j].highway[i]);
-    printf("}%s\n",i==(Way_Unknown-1)?"":", ");
+    printf("}%s\n",i==(Way_Count-1)?"":",");
    }
  printf("   );\n");
  printf("\n");
 
  printf("# Speed limits\n");
  printf("%%router_profile_speed=(\n");
- for(i=1;i<Way_Unknown;i++)
+ for(i=1;i<Way_Count;i++)
    {
     printf("  %12s => {",HighwayName(i));
     for(j=1;j<sizeof(builtin_profiles)/sizeof(builtin_profiles[0]);j++)
        printf("%s %s => %3d",j==1?"":", ",TransportName(j),builtin_profiles[j].speed[i]);
-    printf("}%s\n",i==(Way_Unknown-1)?"":", ");
+    printf("}%s\n",i==(Way_Count-1)?"":",");
+   }
+ printf("   );\n");
+ printf("\n");
+
+ printf("# Highway properties\n");
+ printf("%%router_profile_property=(\n");
+ for(i=1;i<Property_Count;i++)
+   {
+    printf("  %12s => {",PropertyName(i));
+    for(j=1;j<sizeof(builtin_profiles)/sizeof(builtin_profiles[0]);j++)
+       printf("%s %s => %3d",j==1?"":", ",TransportName(j),(int)builtin_profiles[j].props_yes[i]);
+    printf("}%s\n",i==(Property_Count-1)?"":",");
    }
  printf("   );\n");
  printf("\n");
