@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/profiles.h,v 1.9 2009/05/13 18:06:53 amb Exp $
+ $Header: /home/amb/routino/src/RCS/profiles.h,v 1.12 2009/11/25 15:00:37 amb Exp $
 
  A header file for the profiles.
 
@@ -33,23 +33,26 @@
 /*+ A data structure to hold a transport type profile. +*/
 typedef struct _Profile
 {
- Transport transport;           /*+ The type of transport. +*/
+ Transport  transport;                 /*+ The type of transport. +*/
 
- Allowed   allow;               /*+ The type of transport expressed as what must be allowed on a way. +*/
+ wayallow_t allow;                     /*+ The type of transport expressed as what must be allowed on a way. +*/
 
- score_t   highway[Way_Unknown];/*+ A floating point preference for travel on the highway. +*/
- score_t   max_pref;            /*+ The maximum preference for any highway type. +*/
+ score_t    highway[Way_Count];        /*+ A floating point preference for travel on the highway. +*/
+ score_t    max_pref;                  /*+ The maximum preference for any highway type. +*/
 
- speed_t   speed[Way_Unknown];  /*+ The maximum speed on each type of highway. +*/
- speed_t   max_speed;           /*+ The maximum speed for any highway type. +*/
+ speed_t    speed[Way_Count];          /*+ The maximum speed on each type of highway. +*/
+ speed_t    max_speed;                 /*+ The maximum speed for any highway type. +*/
 
- int       oneway;              /*+ A flag to indicate if one-way restrictions apply. +*/
+ score_t    props_yes[Property_Count]; /*+ A floating point preference for ways with this attribute. +*/
+ score_t    props_no [Property_Count]; /*+ A floating point preference for ways without this attribute. +*/
 
- weight_t  weight;              /*+ The minimum weight of the route. +*/
+ int        oneway;                    /*+ A flag to indicate if one-way restrictions apply. +*/
 
- height_t  height;              /*+ The minimum height of vehicles on the route. +*/
- width_t   width;               /*+ The minimum width of vehicles on the route. +*/
- length_t  length;              /*+ The minimum length of vehicles on the route. +*/
+ weight_t   weight;                    /*+ The minimum weight of the route. +*/
+
+ height_t   height;                    /*+ The minimum height of vehicles on the route. +*/
+ width_t    width;                     /*+ The minimum width of vehicles on the route. +*/
+ length_t   length;                    /*+ The minimum length of vehicles on the route. +*/
 }
  Profile;
 
