@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/router.c,v 1.58 2009/08/17 18:26:22 amb Exp $
+ $Header: /home/amb/routino/src/RCS/router.c,v 1.67 2010/01/15 19:48:46 amb Exp $
 
  OSM router.
 
@@ -38,6 +38,18 @@
 /*+ The number of waypoints allowed to be specified. +*/
 #define NWAYPOINTS 99
 
+/*+ The maximum distance from the specified point to search for a node or segment (in km). +*/
+#define MAXSEARCH  5
+
+/*+ The minimum distance along a segment from a node to insert a fake node. (in km). +*/
+#define MINSEGMENT 0.005
+
+
+/*+ A set of fake segments to allow start/finish in the middle of a segment. +*/
+static Segment fake_segments[2*NWAYPOINTS];
+
+/*+ A set of fake node latitudes and longitudes. +*/
+static double point_lon[NWAYPOINTS+1],point_lat[NWAYPOINTS+1];
 
 /*+ The option not to print any progress information. +*/
 int option_quiet=0;
@@ -53,13 +65,13 @@ int main(int argc,char** argv)
  Ways     *OSMWays;
  Results  *results[NWAYPOINTS+1]={NULL};
  int       point_used[NWAYPOINTS+1]={0};
- double    point_lon[NWAYPOINTS+1],point_lat[NWAYPOINTS+1];
  int       help_profile=0,help_profile_js=0,help_profile_pl=0;
  char     *dirname=NULL,*prefix=NULL,*filename;
+ int       exactnodes=0;
  Transport transport=Transport_None;
  Profile   profile;
  index_t   start=NO_NODE,finish=NO_NODE;
- int       arg,node;
+ int       arg,point;
 
  /* Parse the command line arguments */
 
@@ -73,10 +85,12 @@ int main(int argc,char** argv)
                    "              [--help | --help-profile | --help-profile-js | --help-profile-pl]\n"
                    "              [--dir=<name>] [--prefix=<name>]\n"
                    "              [--shortest | --quickest]\n"
+                   "              [--exact-nodes-only]\n"
                    "              [--quiet]\n"
                    "              [--transport=<transport>]\n"
                    "              [--highway-<highway>=<preference> ...]\n"
                    "              [--speed-<highway>=<speed> ...]\n"
+                   "              [--property-<property>=<preference> ...]\n"
                    "              [--oneway=[0|1]]\n"
                    "              [--weight=<weight>]\n"
                    "              [--height=<height>] [--width=<width>] [--length=<length>]\n"
@@ -87,12 +101,15 @@ int main(int argc,char** argv)
                    "<highway> can be selected from:\n"
                    "%s"
                    "\n"
+                   "<property> can be selected from:\n"
+                   "%s"
+                   "\n"
                    "<preference> is a preference expressed as a percentage\n"
                    "<speed> is a speed in km/hour\n"
                    "<weight> is a weight in tonnes\n"
                    "<height>, <width>, <length> are dimensions in metres\n"
                    "\n",
-                   TransportList(),HighwayList());
+            TransportList(),HighwayList(),PropertyList());
 
     return(1);
    }
@@ -120,18 +137,18 @@ int main(int argc,char** argv)
     if(isdigit(argv[arg][0]) ||
        ((argv[arg][0]=='-' || argv[arg][0]=='+') && isdigit(argv[arg][1])))
       {
-       for(node=1;node<=NWAYPOINTS;node++)
-          if(point_used[node]!=3)
+       for(point=1;point<=NWAYPOINTS;point++)
+          if(point_used[point]!=3)
             {
-             if(point_used[node]==0)
+             if(point_used[point]==0)
                {
-                point_lon[node]=degrees_to_radians(atof(argv[arg]));
-                point_used[node]=1;
+                point_lon[point]=degrees_to_radians(atof(argv[arg]));
+                point_used[point]=1;
                }
-             else /* if(point_used[node]==1) */
+             else /* if(point_used[point]==1) */
                {
-                point_lat[node]=degrees_to_radians(atof(argv[arg]));
-                point_used[node]=3;
+                point_lat[point]=degrees_to_radians(atof(argv[arg]));
+                point_used[point]=3;
                }
              break;
             }
@@ -143,12 +160,12 @@ int main(int argc,char** argv)
         if(*p++!='=')
            goto usage;
  
-        node=atoi(&argv[arg][5]);
-        if(node>NWAYPOINTS || point_used[node]&1)
+        point=atoi(&argv[arg][5]);
+        if(point>NWAYPOINTS || point_used[point]&1)
            goto usage;
  
-       point_lon[node]=degrees_to_radians(atof(p));
-       point_used[node]+=1;
+       point_lon[point]=degrees_to_radians(atof(p));
+       point_used[point]+=1;
       }
      else if(!strncmp(argv[arg],"--lat",5) && isdigit(argv[arg][5]))
        {
@@ -157,12 +174,12 @@ int main(int argc,char** argv)
         if(*p++!='=')
            goto usage;
  
-        node=atoi(&argv[arg][5]);
-        if(node>NWAYPOINTS || point_used[node]&2)
+        point=atoi(&argv[arg][5]);
+        if(point>NWAYPOINTS || point_used[point]&2)
            goto usage;
  
-       point_lat[node]=degrees_to_radians(atof(p));
-       point_used[node]+=2;
+       point_lat[point]=degrees_to_radians(atof(p));
+       point_used[point]+=2;
       }
     else if(!strcmp(argv[arg],"--help"))
        goto usage;
@@ -180,6 +197,8 @@ int main(int argc,char** argv)
        option_quickest=0;
     else if(!strcmp(argv[arg],"--quickest"))
        option_quickest=1;
+    else if(!strcmp(argv[arg],"--exact-nodes-only"))
+       exactnodes=1;
     else if(!strcmp(argv[arg],"--quiet"))
        option_quiet=1;
     else if(!strncmp(argv[arg],"--transport=",12))
@@ -198,12 +217,12 @@ int main(int argc,char** argv)
 
        highway=HighwayType(string);
 
-       free(string);
-
-       if(highway==Way_Unknown)
+       if(highway==Way_Count)
           goto usage;
 
        profile.highway[highway]=atof(equal+1);
+
+       free(string);
       }
     else if(!strncmp(argv[arg],"--speed-",8))
       {
@@ -219,12 +238,33 @@ int main(int argc,char** argv)
 
        highway=HighwayType(string);
 
-       free(string);
-
-       if(highway==Way_Unknown)
+       if(highway==Way_Count)
           goto usage;
 
        profile.speed[highway]=kph_to_speed(atof(equal+1));
+
+       free(string);
+      }
+    else if(!strncmp(argv[arg],"--property-",11))
+      {
+       Property property;
+       char *equal=strchr(argv[arg],'=');
+       char *string;
+
+       if(!equal)
+          goto usage;
+
+       string=strcpy((char*)malloc(strlen(argv[arg])),argv[arg]+11);
+       string[equal-argv[arg]-11]=0;
+
+       property=PropertyType(string);
+
+       if(property==Way_Count)
+          goto usage;
+
+       profile.props_yes[property]=atof(equal+1);
+
+       free(string);
       }
     else if(!strncmp(argv[arg],"--oneway=",9))
        profile.oneway=!!atoi(&argv[arg][9]);
@@ -240,8 +280,8 @@ int main(int argc,char** argv)
        goto usage;
    }
 
- for(node=0;node<=NWAYPOINTS;node++)
-    if(point_used[node]==1 || point_used[node]==2)
+ for(point=1;point<=NWAYPOINTS;point++)
+    if(point_used[point]==1 || point_used[point]==2)
        goto usage;
 
  if(help_profile)
@@ -291,25 +331,45 @@ int main(int argc,char** argv)
     return(1);
    }
 
- /* Loop through all pairs of nodes */
+ if(!(profile.allow & OSMWays->allow))
+   {
+    fprintf(stderr,"Error: Database was not generated for selected transport.\n");
+    return(1);
+   }
 
- for(node=1;node<=NWAYPOINTS;node++)
+ /* Loop through all pairs of points */
+
+ for(point=1;point<=NWAYPOINTS;point++)
    {
     Results *begin,*end;
-    distance_t dist=km_to_distance(10);
+    distance_t distmax=km_to_distance(MAXSEARCH);
+    distance_t distmin;
+    Segment *segment=NULL;
+    index_t node1,node2;
 
-    if(point_used[node]!=3)
+    if(point_used[point]!=3)
        continue;
 
-    /* Find the node */
+    /* Find the closest point */
 
     start=finish;
 
-    finish=FindNode(OSMNodes,OSMSegments,OSMWays,point_lat[node],point_lon[node],&dist,&profile);
+    if(exactnodes)
+      {
+       finish=FindClosestNode(OSMNodes,OSMSegments,OSMWays,point_lat[point],point_lon[point],distmax,&profile,&distmin);
+      }
+    else
+      {
+       distance_t dist1,dist2;
+
+       segment=FindClosestSegment(OSMNodes,OSMSegments,OSMWays,point_lat[point],point_lon[point],distmax,&profile,&distmin,&node1,&node2,&dist1,&dist2);
+
+       finish=CreateFakes(OSMNodes,point,segment,node1,node2,dist1,dist2);
+      }
 
     if(finish==NO_NODE)
       {
-       fprintf(stderr,"Error: Cannot find node close to specified point %d.\n",node);
+       fprintf(stderr,"Error: Cannot find node close to specified point %d.\n",point);
        return(1);
       }
 
@@ -317,9 +377,17 @@ int main(int argc,char** argv)
       {
        double lat,lon;
 
-       GetLatLong(OSMNodes,finish,&lat,&lon);
+       if(IsFakeNode(finish))
+          GetFakeLatLong(finish,&lat,&lon);
+       else
+          GetLatLong(OSMNodes,finish,&lat,&lon);
 
-       printf("Node %d: %3.6f %4.6f = %2.3f km\n",node,radians_to_degrees(lon),radians_to_degrees(lat),distance_to_km(dist));
+       if(IsFakeNode(finish))
+          printf("Point %d is segment %d (node %d -> %d): %3.6f %4.6f = %2.3f km\n",point,IndexSegment(OSMSegments,segment),node1,node2,
+                 radians_to_degrees(lon),radians_to_degrees(lat),distance_to_km(distmin));
+       else
+          printf("Point %d is node %d: %3.6f %4.6f = %2.3f km\n",point,finish,
+                 radians_to_degrees(lon),radians_to_degrees(lat),distance_to_km(distmin));
       }
 
     if(start==NO_NODE)
@@ -327,7 +395,7 @@ int main(int argc,char** argv)
 
     /* Calculate the beginning of the route */
 
-    if(IsSuperNode(OSMNodes,start))
+    if(!IsFakeNode(start) && IsSuperNode(OSMNodes,start))
       {
        Result *result;
 
@@ -352,9 +420,15 @@ int main(int argc,char** argv)
 
     if(FindResult(begin,finish))
       {
-       results[node]=begin;
+       FixForwardRoute(begin,finish);
 
-       results[node]->finish=finish;
+       results[point]=begin;
+
+       if(!option_quiet)
+         {
+          printf("\rRouted: Super-Nodes Checked = %d\n",begin->number);
+          fflush(stdout);
+         }
       }
     else
       {
@@ -362,7 +436,7 @@ int main(int argc,char** argv)
 
        /* Calculate the end of the route */
 
-       if(IsSuperNode(OSMNodes,finish))
+       if(!IsFakeNode(finish) && IsSuperNode(OSMNodes,finish))
          {
           Result *result;
 
@@ -389,13 +463,18 @@ int main(int argc,char** argv)
 
        superresults=FindMiddleRoute(OSMNodes,OSMSegments,OSMWays,begin,end,&profile);
 
+       FreeResultsList(begin);
+       FreeResultsList(end);
+
        if(!superresults)
          {
           fprintf(stderr,"Error: Cannot find route compatible with profile.\n");
           return(1);
          }
 
-       results[node]=CombineRoutes(superresults,OSMNodes,OSMSegments,OSMWays,&profile);
+       results[point]=CombineRoutes(superresults,OSMNodes,OSMSegments,OSMWays,&profile);
+
+       FreeResultsList(superresults);
       }
    }
 
@@ -403,11 +482,173 @@ int main(int argc,char** argv)
 
  PrintRouteHead(FileName(dirname,prefix,"copyright.txt"));
 
- for(node=1;node<=NWAYPOINTS;node++)
-    if(results[node])
-       PrintRoute(results[node],OSMNodes,OSMSegments,OSMWays,&profile);
+ PrintRoute(results,NWAYPOINTS,OSMNodes,OSMSegments,OSMWays,&profile);
 
  PrintRouteTail();
 
  return(0);
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Create a pair of fake segments corresponding to the given segment split in two.
+
+  index_t CreateFakes Returns the fake node index (or a real one in special cases).
+
+  Nodes *nodes The set of nodes to use.
+
+  int point Which of the waypoints is this.
+
+  Segment *segment The segment to split.
+
+  index_t node1 The first node at the end of this segment.
+
+  index_t node2 The second node at the end of this segment.
+
+  distance_t dist1 The distance to the first node.
+
+  distance_t dist2 The distance to the second node.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+index_t CreateFakes(Nodes *nodes,int point,Segment *segment,index_t node1,index_t node2,distance_t dist1,distance_t dist2)
+{
+ index_t fakenode;
+ double lat1,lon1,lat2,lon2;
+
+ /* Check if we are actually close enough to an existing node */
+
+ if(dist1<km_to_distance(MINSEGMENT) && dist2>km_to_distance(MINSEGMENT))
+    return(node1);
+
+ if(dist2<km_to_distance(MINSEGMENT) && dist1>km_to_distance(MINSEGMENT))
+    return(node2);
+
+ if(dist1<km_to_distance(MINSEGMENT) && dist2<km_to_distance(MINSEGMENT))
+   {
+    if(dist1<dist2)
+       return(node1);
+    else
+       return(node2);
+   }
+
+ /* Create the fake node */
+
+ fakenode=point|NODE_SUPER;
+
+ GetLatLong(nodes,node1,&lat1,&lon1);
+ GetLatLong(nodes,node2,&lat2,&lon2);
+
+ if(lat1>3 && lat2<-3)
+    lat2+=2*M_PI;
+ else if(lat1<-3 && lat2>3)
+    lat1+=2*M_PI;
+
+ point_lat[point]=lat1+(lat2-lat1)*(double)dist1/(double)(dist1+dist2);
+ point_lon[point]=lon1+(lon2-lon1)*(double)dist1/(double)(dist1+dist2);
+
+ if(point_lat[point]>M_PI) point_lat[point]-=2*M_PI;
+
+ /* Create the first fake segment */
+
+ fake_segments[2*point-2]=*segment;
+
+ if(segment->node1==node1)
+    fake_segments[2*point-2].node1=fakenode;
+ else
+    fake_segments[2*point-2].node2=fakenode;
+
+ fake_segments[2*point-2].distance=DISTANCE(dist1)|DISTFLAG(segment->distance);
+
+ /* Create the second fake segment */
+
+ fake_segments[2*point-1]=*segment;
+
+ if(segment->node1==node2)
+    fake_segments[2*point-1].node1=fakenode;
+ else
+    fake_segments[2*point-1].node2=fakenode;
+
+ fake_segments[2*point-1].distance=DISTANCE(dist2)|DISTFLAG(segment->distance);
+
+ return(fakenode);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Lookup the latitude and longitude of a fake node.
+
+  index_t fakenode The node to lookup.
+
+  double *latitude Returns the latitude
+
+  double *longitude Returns the longitude.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void GetFakeLatLong(index_t fakenode, double *latitude,double *longitude)
+{
+ index_t realnode=fakenode&(~NODE_SUPER);
+
+ *latitude =point_lat[realnode];
+ *longitude=point_lon[realnode];
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Finds the first fake segment associated to a fake node.
+
+  Segment *FirstFakeSegment Returns the first fake segment.
+
+  index_t fakenode The node to lookup.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+Segment *FirstFakeSegment(index_t fakenode)
+{
+ index_t realnode=fakenode&(~NODE_SUPER);
+
+ return(&fake_segments[2*realnode-2]);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Finds the next (there can only be two) fake segment associated to a fake node.
+
+  Segment *NextFakeSegment Returns the second fake segment.
+
+  Segment *segment The first fake segment.
+
+  index_t fakenode The node to lookup.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+Segment *NextFakeSegment(Segment *segment,index_t fakenode)
+{
+ index_t realnode=fakenode&(~NODE_SUPER);
+
+ if(segment==&fake_segments[2*realnode-2])
+    return(&fake_segments[2*realnode-1]);
+ else
+    return(NULL);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Finds the next (there can only be two) fake segment associated to a fake node.
+
+  Segment *ExtraFakeSegment Returns a segment between the two specified nodes if it exists.
+
+  index_t node The real node.
+
+  index_t fakenode The fake node to lookup.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+Segment *ExtraFakeSegment(index_t node,index_t fakenode)
+{
+ index_t realnode=fakenode&(~NODE_SUPER);
+
+ if(fake_segments[2*realnode-2].node1==node || fake_segments[2*realnode-2].node2==node)
+    return(&fake_segments[2*realnode-2]);
+
+ if(fake_segments[2*realnode-1].node1==node || fake_segments[2*realnode-1].node2==node)
+    return(&fake_segments[2*realnode-1]);
+
+ return(NULL);
+}
