@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/segments.c,v 1.38 2009/05/14 18:02:30 amb Exp $
+ $Header: /home/amb/routino/src/RCS/segments.c,v 1.43 2009/07/12 09:01:48 amb Exp $
 
  Segment data type functions.
 
@@ -22,9 +22,7 @@
  ***************************************/
 
 
-#include <assert.h>
-#include <math.h>
-#include <string.h>
+#include <sys/types.h>
 #include <stdlib.h>
 
 #include "types.h"
@@ -38,7 +36,7 @@
 /*++++++++++++++++++++++++++++++++++++++
   Load in a segment list from a file.
 
-  Segments* SaveSegmentList Returns the segment list that has just been loaded.
+  Segments* LoadSegmentList Returns the segment list that has just been loaded.
 
   const char *filename The name of the file to load.
   ++++++++++++++++++++++++++++++++++++++*/
@@ -76,14 +74,16 @@ Segments *LoadSegmentList(const char *filename)
   Segments* segments The set of segments to process.
 
   Segment *segment The current segment.
+
+  index_t node The current node.
   ++++++++++++++++++++++++++++++++++++++*/
 
 Segment *NextSegment(Segments* segments,Segment *segment,index_t node)
 {
- if(NODE(segment->node1)==node)
+ if(segment->node1==node)
    {
     segment++;
-    if(NODE(segment->node1)!=node || (segment-segments->segments)>=segments->number)
+    if((segment-segments->segments)>=segments->number || segment->node1!=node)
        return(NULL);
     else
        return(segment);
@@ -103,33 +103,33 @@ Segment *NextSegment(Segments* segments,Segment *segment,index_t node)
 
   distance_t Distance Returns the distance between the locations.
 
-  float lat1 The latitude of the first location.
+  double lat1 The latitude of the first location.
 
-  float lon1 The longitude of the first location.
+  double lon1 The longitude of the first location.
 
-  float lat2 The latitude of the second location.
+  double lat2 The latitude of the second location.
 
-  float lon2 The longitude of the second location.
+  double lon2 The longitude of the second location.
   ++++++++++++++++++++++++++++++++++++++*/
 
-distance_t Distance(float lat1,float lon1,float lat2,float lon2)
+distance_t Distance(double lat1,double lon1,double lat2,double lon2)
 {
- float dlon = lon1 - lon2;
- float dlat = lat1 - lat2;
+ double dlon = lon1 - lon2;
+ double dlat = lat1 - lat2;
 
- float a1,a2,a,sa,c,d;
+ double a1,a2,a,sa,c,d;
 
  if(dlon==0 && dlat==0)
    return 0;
 
- a1 = sinf (dlat / 2);
- a2 = sinf (dlon / 2);
- a = (a1 * a1) + cosf (lat1) * cosf (lat2) * a2 * a2;
- sa = sqrtf (a);
+ a1 = sin (dlat / 2);
+ a2 = sin (dlon / 2);
+ a = (a1 * a1) + cos (lat1) * cos (lat2) * a2 * a2;
+ sa = sqrt (a);
  if (sa <= 1.0)
-   {c = 2 * asinf (sa);}
+   {c = 2 * asin (sa);}
  else
-   {c = 2 * asinf (1.0);}
+   {c = 2 * asin (1.0);}
  d = 6378.137 * c;
 
  return km_to_distance(d);
