@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/segmentsx.c,v 1.45 2009/10/12 17:54:18 amb Exp $
+ $Header: /home/amb/routino/src/RCS/segmentsx.c,v 1.48 2009/12/12 11:08:50 amb Exp $
 
  Extended Segment data type functions.
 
@@ -38,13 +38,12 @@
 #include "ways.h"
 
 
-/* Constants */
-
-#define SORT_RAMSIZE (64*1024*1024)
-
 /* Variables */
 
+/*+ The command line '--slim' option. +*/
 extern int option_slim;
+
+/*+ The command line '--tmpdir' option or its default value. +*/
 extern char *option_tmpdirname;
 
 /* Local Functions */
@@ -162,7 +161,7 @@ void SortSegmentList(SegmentsX* segmentsx)
 
  /* Sort by node indexes */
 
- filesort(segmentsx->fd,fd,sizeof(SegmentX),SORT_RAMSIZE,(int (*)(const void*,const void*))sort_by_id,NULL);
+ filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),(int (*)(const void*,const void*))sort_by_id,NULL);
 
  segmentsx->number=segmentsx->xnumber;
 
@@ -803,7 +802,7 @@ void CreateRealSegments(SegmentsX *segmentsx,WaysX *waysx)
     segmentsx->sdata[i].node1=0;
     segmentsx->sdata[i].node2=0;
     segmentsx->sdata[i].next2=NO_NODE;
-    segmentsx->sdata[i].way=wayx->id;
+    segmentsx->sdata[i].way=wayx->prop;
     segmentsx->sdata[i].distance=segmentx->distance;
 
     if(!((i+1)%10000))
