@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/ways.h,v 1.31 2009/09/17 12:53:58 amb Exp $
+ $Header: /home/amb/routino/src/RCS/ways.h,v 1.34 2009/11/25 15:00:37 amb Exp $
 
  A header file for the ways.
 
@@ -33,36 +33,39 @@
 /* Data structures */
 
 
-/*+ A structure containing a single way. +*/
+/*+ A structure containing a single way (members ordered to minimise overall size). +*/
 struct _Way
 {
  index_t    name;               /*+ The offset of the name of the way in the names array. +*/
 
- waytype_t  type;               /*+ The type of the way. +*/
-
  wayallow_t allow;              /*+ The type of traffic allowed on the way. +*/
 
+ waytype_t  type;               /*+ The highway type of the way. +*/
+
+ wayprop_t  props;              /*+ The properties of the way. +*/
+
  speed_t    speed;              /*+ The defined maximum speed limit of the way. +*/
 
  weight_t   weight;             /*+ The defined maximum weight of traffic on the way. +*/
  height_t   height;             /*+ The defined maximum height of traffic on the way. +*/
  width_t    width;              /*+ The defined maximum width of traffic on the way. +*/
  length_t   length;             /*+ The defined maximum length of traffic on the way. +*/
-
- uint8_t    padding;            /*+ Padding to make a multiple of 4 bytes +*/
 };
 
 
 /*+ A structure containing a set of ways (mmap format). +*/
 struct _Ways
 {
- uint32_t number;               /*+ How many ways are stored? +*/
- uint32_t onumber;              /*+ How many ways were there originally? +*/
+ uint32_t   number;             /*+ How many ways are stored? +*/
+ uint32_t   onumber;            /*+ How many ways were there originally? +*/
 
- Way     *ways;                 /*+ An array of ways. +*/
- char    *names;                /*+ An array of characters containing the names. +*/
+ wayallow_t allow;              /*+ The type of traffic allowed when parsing. +*/
+ wayprop_t  props;              /*+ The properties that were included when parsing. +*/
 
- void    *data;                 /*+ The memory mapped data. +*/
+ Way       *ways;               /*+ An array of ways. +*/
+ char      *names;              /*+ An array of characters containing the names. +*/
+
+ void      *data;               /*+ The memory mapped data. +*/
 };
 
 
@@ -83,12 +86,18 @@ Ways *LoadWayList(const char *filename);
 
 Highway HighwayType(const char *highway);
 Transport TransportType(const char *transport);
+Property PropertyType(const char *property);
 
 const char *HighwayName(Highway highway);
 const char *TransportName(Transport transport);
+const char *PropertyName(Property property);
+
+const char *AllowedNameList(wayallow_t allowed);
+const char *PropertiesNameList(wayprop_t properties);
 
 const char *HighwayList(void);
 const char *TransportList(void);
+const char *PropertyList(void);
 
 int WaysCompare(Way *way1,Way *way2);
 
